import 'package:fis_jsonrpc/rpc.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:intl/intl.dart';
import 'package:vitalapp/architecture/utils/datetime.dart';
import 'package:vitalapp/architecture/utils/sensitive.dart';
import 'package:vitalapp/components/button.dart';
import 'package:vitalapp/components/dialog_date.dart';
import 'package:vitalapp/components/dynamic_drawer.dart';
import 'package:vitalapp/components/input.dart';
import 'package:vitalapp/components/search_input.dart';
import 'package:vitalapp/consts/rpc_enum_labels.dart';
import 'package:vitalapp/consts/styles.dart';
import 'package:vitalapp/managers/contract/index.dart';
import 'package:vitalapp/pages/patient/create/widgets/quick_create.dart';
import 'package:vitalapp/pages/patient/list/widgets/status.dart';
import 'package:vitalapp/pages/patient/list/widgets/tab_button.dart';
import 'package:vitalapp/pages/patient/list/widgets/tab_button_group.dart';
import 'package:vitalapp/store/store.dart';

import 'controller.dart';

class PatientListPage extends GetView<PatientListController> {
  const PatientListPage({super.key});

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: Colors.transparent,
      body: Stack(
        children: [
          Container(
            padding: const EdgeInsets.all(16),
            decoration: BoxDecoration(
              color: Colors.grey.shade200,
              borderRadius: BorderRadius.circular(
                16,
              ),
              border: Border.all(),
            ),
            child: Column(
              mainAxisSize: MainAxisSize.max,
              children: [
                Obx(
                  () => TabButtonGroup(
                    [
                      TabButton(
                        tabTitle: '新建档案',
                        index: 0,
                        activeIndex: controller.state.currentTabIndex,
                        onCallbackTap: () {
                          controller.state.currentTabIndex = 0;
                        },
                        width: 150,
                      ),
                      TabButton(
                        tabTitle: '居民列表',
                        index: 1,
                        activeIndex: controller.state.currentTabIndex,
                        onCallbackTap: () {
                          controller.state.currentTabIndex = 1;
                        },
                        width: 150,
                      ),
                    ],
                  ),
                ),
                const SizedBox(
                  height: 8,
                ),
                _buildContent(context),
              ],
            ),
          ),
          Positioned(
            right: 8,
            top: 8,
            child: IconButton(
              onPressed: () {
                Get.back();
              },
              icon: const Icon(
                Icons.close,
                size: 30,
              ),
            ),
          ),
        ],
      ),
    );
  }

  Widget _buildContent(BuildContext context) {
    return Obx(
      () {
        if (controller.state.currentTabIndex == 0) {
          return Expanded(
            child: Container(
              color: Colors.white,
              child: const QuickCreatePatientPage(),
            ),
          );
        } else {
          return Expanded(
            child: Column(
              children: [
                _HeaderWidget(
                  onFilterPressed: () {
                    VDynamicDrawerWrapper.show(
                      scaffoldKey: controller.scaffoldKey,
                      builder: (_) => _filterdrawer(context),
                    );
                    // scaffoldKey.currentState?.openEndDrawer();
                  },
                ),
                const SizedBox(height: 20),
                Expanded(child: _buildListView()),
              ],
            ),
          );
        }
      },
    );
  }

  VDrawer _filterdrawer(BuildContext context) {
    final scrollController = ScrollController();
    controller.crowdLabelsController.onReady();
    return VDrawer(
      width: 600,
      title: "筛选",
      scaffoldKey: controller.scaffoldKey,
      onConfirm: () {
        controller.reloadList(isFilter: true);
        // Get.back();
        VDynamicDrawerWrapper.hide(
          scaffoldKey: controller.scaffoldKey,
        );
      },
      onCancel: () {
        // Get.back();
        VDynamicDrawerWrapper.hide(
          scaffoldKey: controller.scaffoldKey,
        );
      },
      child: Scrollbar(
        controller: scrollController,
        thumbVisibility: true,
        child: SingleChildScrollView(
          controller: scrollController,
          child: Padding(
            padding: const EdgeInsets.symmetric(horizontal: 40, vertical: 20),
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                const Text(
                  '居民创建时间：',
                  style: TextStyle(fontSize: 20),
                ),
                const SizedBox(
                  height: 20,
                ),
                Row(
                  mainAxisAlignment: MainAxisAlignment.spaceBetween,
                  mainAxisSize: MainAxisSize.max,
                  children: [
                    Expanded(
                      child: Obx(
                        () => VInput(
                          readOnly: true,
                          controller: TextEditingController(
                            text: DateFormat('yyyy-MM-dd').format(
                              controller.state.startTime.value!.toLocal(),
                            ),
                          ),
                          radius: 4,
                          onTap: () async {
                            final result = await VDialogDate(
                              maxValue: controller.state.endTime.value,
                              title: '起始时间',
                              initialValue: controller.state.startTime.value,
                            ).show();
                            controller.state.startTime.value = result;
                          },
                        ),
                      ),
                    ),
                    Container(
                        margin: const EdgeInsets.symmetric(horizontal: 16),
                        child: const Text('一')),
                    Expanded(
                      child: Obx(
                        () => VInput(
                          readOnly: true,
                          controller: TextEditingController(
                            text: DateFormat('yyyy-MM-dd').format(
                              controller.state.endTime.value!.toLocal(),
                            ),
                          ),
                          radius: 4,
                          onTap: () async {
                            final result = await VDialogDate(
                              title: '结束时间',
                              initialValue: controller.state.endTime.value,
                            ).show();
                            controller.state.endTime.value = result;
                          },
                        ),
                      ),
                    )
                  ],
                ),
                // const SizedBox(
                //   height: 20,
                // ),
                // const Text(
                //   '人群分类:',
                //   style: TextStyle(fontSize: 20),
                // ),
                // const SizedBox(
                //   height: 20,
                // ),
                // CrowdSelectLabelView(
                //   controller: controller.crowdLabelsController,
                // ),
              ],
            ),
          ),
        ),
      ),
    );
  }

  Widget _buildListView() {
    final scrollController = ScrollController();
    scrollController.addListener(
      () {
        // 如果滑动到底部
        try {
          if (scrollController.position.atEdge) {
            if (scrollController.position.pixels != 0) {
              if (controller.state.hasNextPage) {
                controller.loadNextPageList();
              }
            }
          }
        } catch (e) {
          // logger.e("listViewScrollController exception:", e);
        }
      },
    );
    return RefreshIndicator(
      onRefresh: () async {
        controller.reloadList();
      },
      child: Obx(
        () {
          final list = controller.state.dataList;
          final children = <Widget>[];
          for (var i = 0; i < list.length; i++) {
            children.add(_PatientCard(dto: list[i]));
          }
          return Scrollbar(
            trackVisibility: true,
            controller: scrollController,
            child: GridView(
              shrinkWrap: true,
              controller: scrollController,
              gridDelegate: const SliverGridDelegateWithFixedCrossAxisCount(
                crossAxisCount: 3,
                mainAxisSpacing: 16,
                crossAxisSpacing: 20,
                childAspectRatio: 360 / 180,
              ),
              children: children,
            ),
          );
        },
      ),
    );
  }
}

class _HeaderWidget extends GetView<PatientListController> {
  final VoidCallback onFilterPressed;

  const _HeaderWidget({
    required this.onFilterPressed,
  });
  @override
  Widget build(BuildContext context) {
    return SizedBox(
      height: 76,
      child: Row(
        children: [
          Expanded(
            child: SizedBox(
              height: 70,
              child: VSearchInput(
                placeholder: "身份证号码/姓名/手机号",
                onSearch: (value) {
                  controller.state.searchString = value;
                  controller.reloadList();
                },
              ),
            ),
          ),
          const SizedBox(width: 8),
          SizedBox(
            width: 180,
            height: 70,
            child: VButton(
              onTap: onFilterPressed,
              child: Row(
                mainAxisAlignment: MainAxisAlignment.center,
                children: const [
                  Icon(Icons.filter_alt, size: 24),
                  Text("筛选", style: TextStyle(fontSize: 20)),
                ],
              ),
            ),
          ),
        ],
      ),
    );
  }
}

class _PatientCard extends StatelessWidget {
  final PatientDTO dto;

  const _PatientCard({required this.dto});

  @override
  Widget build(BuildContext context) {
    final body = Stack(
      children: [
        Container(
          padding: const EdgeInsets.symmetric(
            horizontal: 16,
            vertical: 12,
          ),
          child: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              const SizedBox(height: 8),
              SizedBox(
                child: Text(
                  dto.patientName!,
                  overflow: TextOverflow.ellipsis,
                  style: const TextStyle(
                    color: Colors.black,
                    fontSize: 26,
                    fontWeight: FontWeight.bold,
                  ),
                ),
              ),
              const SizedBox(height: 8),
              LayoutBuilder(
                builder: (context, c) {
                  final width = c.maxWidth - 80;
                  // 不和状态标签重叠，并保持一定距离
                  return SizedBox(width: width, child: _buildBaseInfoRow());
                },
              ),
              const SizedBox(height: 12),
              // Expanded(child: _buildClassTags()),
              // _buildClassTags(),
              const SizedBox(height: 6),
              _buildPhone(),
              const SizedBox(height: 6),
              _buildCardNo(),
            ],
          ),
        ),
        // Positioned(
        //   top: 0,
        //   right: 0,
        //   child: _PatientSignStatusTag(
        //     dto: dto,
        //   ),
        // ),
      ],
    );
    return Material(
      borderRadius: GlobalStyles.borderRadius,
      child: Ink(
        decoration: BoxDecoration(
          color: Colors.white,
          borderRadius: GlobalStyles.borderRadius,
        ),
        child: InkWell(
          borderRadius: GlobalStyles.borderRadius,
          onTap: () {
            // Get.back();
            Get.find<PatientListController>().gotoDetail(dto.code!);
          },
          child: body,
        ),
      ),
    );
  }

  Widget _buildBaseInfoRow() {
    final birthday = dto.birthday!.toLocal();
    final age = DataTimeUtils.calculateAge(birthday);
    return Row(
      mainAxisSize: MainAxisSize.max,
      mainAxisAlignment: MainAxisAlignment.spaceBetween,
      children: [
        Expanded(
          child: Row(
            mainAxisSize: MainAxisSize.max,
            mainAxisAlignment: MainAxisAlignment.start,
            children: [
              Expanded(
                child: Text(
                  RpcEnumLabels.gender[dto.patientGender] ?? "未知",
                  style: const TextStyle(
                    color: Colors.grey,
                    fontSize: 20,
                    fontWeight: FontWeight.bold,
                  ),
                ),
              ),
              Expanded(
                child: Text(
                  "$age岁",
                  style: const TextStyle(
                    color: Colors.grey,
                    fontSize: 20,
                  ),
                ),
              ),
            ],
          ),
        ),
      ],
    );
  }

  Widget _buildClassTags() {
    return Column(
      children: [
        ConstrainedBox(
          constraints: const BoxConstraints(
            minWidth: double.infinity,
            maxHeight: 50,
          ),
          child: Text(
            dto.labelNames?.join('   ') ?? '',
            style: const TextStyle(color: Colors.grey, fontSize: 18),
            maxLines: 2,
            overflow: TextOverflow.ellipsis,
          ),
        ),
      ],
    );
  }

  Widget _buildPhone() {
    if (dto.phone != null && dto.phone!.isNotEmpty) {
      String phone = dto.phone!;
      if (Store.app.enableEncryptSensitiveInfo) {
        phone = SensitiveUtils.desensitizeMobilePhone(phone);
      }
      return Text(
        '手机号：$phone',
        style: const TextStyle(color: Colors.grey, fontSize: 18),
      );
    } else {
      return const SizedBox();
    }
  }

  Widget _buildCardNo() {
    if (dto.cardNo != null && dto.cardNo!.isNotEmpty) {
      String cardNo = dto.cardNo!;
      if (Store.app.enableEncryptSensitiveInfo) {
        cardNo = SensitiveUtils.desensitizeIdCard(cardNo);
      }
      return Text(
        '证件号码：$cardNo',
        style: const TextStyle(color: Colors.grey, fontSize: 18),
      );
    } else {
      return const SizedBox();
    }
  }
}

class _PatientSignStatusTag extends StatelessWidget {
  final PatientDTO dto;
  _PatientSignStatusTag({required this.dto});
  final ContractUtils _contractUtils = ContractUtils();

  @override
  Widget build(BuildContext context) {
    return Container(
      alignment: Alignment.centerRight,
      width: 120,
      padding: const EdgeInsets.only(top: 18),
      child: StatusLabel(
        title: _contractUtils.contractStateTransition(dto.contractState),
        color: _contractUtils.contractStateColors(dto.contractState),
      ),
    );
  }
}
